'''
Created on 05.01.2017

@author: Lutz
'''
from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg
from dateutil.relativedelta import relativedelta
from matplotlib.figure import Figure
from tkinter import DISABLED
from sys import path
import matplotlib.dates as mdates
import tkinter.scrolledtext
import serial, time, datetime
import mytools, json
import tkinter.messagebox
import sqlite3

def firstSqlInit():
    connection = sqlite3.connect('wetterdaten.db')  
    cursor = connection.cursor()
    cursor.execute("""CREATE TABLE wetter(datetime TEXT, temperatur REAL, feuchte REAL, druck REAL, beobachtung INTEGER)""")

def loadSetup():
    try:
        with open('config.json', 'r') as f:
            config = json.load(f)
            f.close()
    except IOError:
        config = {'key1': path[0]+"\wetterdaten.db", 'key2': "COM15"}
        with open('config.json', 'w') as f:
            json.dump(config, f)
            f.close()            
    PathStr = config['key1']
    COMStr = config['key2']
    return PathStr, COMStr


class WetterApp(tkinter.Frame):
    def __init__(self, master):
        super().__init__(master)
        
        self.PathStr, self.COMStr = loadSetup()

        self.menuBar = tkinter.Menu(master)
        master.config(menu=self.menuBar)
        self.fillMenuBar()
        
        self.text = tkinter.scrolledtext.ScrolledText(master, wrap=tkinter.WORD, width=40)
        self.text.grid(row=0,column=0,padx=0, pady=0)
        
        buttonGraph = tkinter.Button(master, text='Aktuell', command=self.cmdAktuell)
        buttonGraph.grid(row=1, column=1, padx=5, pady=15, sticky=tkinter.W)
        buttonRuck = tkinter.Button(master, text='<<', command=self.cmdSkipRueck)
        buttonRuck.grid(row=1, column=1, padx=65, pady=15, sticky=tkinter.W)
        buttonVor = tkinter.Button(master, text='>>', command=self.cmdSkipVor)
        buttonVor.grid(row=1, column=1, padx=100, pady=15, sticky=tkinter.W)
        buttonBeschreibung = tkinter.Button(master, text='Beschreibung', command=self.cmdBeschreibung)
        buttonBeschreibung.grid(row=1, column=1, padx=705, pady=0, sticky=tkinter.W)
        tkinter.Label(master, text="Startdatum:").grid(row=1,column=1,padx=150, pady=15, sticky=tkinter.W)
        self.eStart = tkinter.Entry(master, width=15)
        self.eStart.grid(row=1,column=1,padx=220, pady=15, sticky=tkinter.W)
        tkinter.Label(master, text="Enddatum:").grid(row=1,column=1,padx=330, pady=15, sticky=tkinter.W)
        self.eEnd = tkinter.Entry(master, width=15)
        self.eEnd.grid(row=1,column=1,padx=395, pady=15, sticky=tkinter.W)
        buttonOk = tkinter.Button(master, text='Ok', command=self.cmdDatumOk)
        buttonOk.grid(row=1, column=1, padx=500, pady=15, sticky=tkinter.W)
        self.iTagStunde = tkinter.IntVar()
        self.iTagStunde.set(1) 
        radio = tkinter.Radiobutton(master, text='Stunde', variable = self.iTagStunde, value = 1, command=self.cmdRadio)
        radio.grid(row=1, column=1, padx=535, pady=15, sticky=tkinter.W)
        radio = tkinter.Radiobutton(master, text='Tag', variable = self.iTagStunde, value = 0, command=self.cmdRadio)
        radio.grid(row=1, column=1, padx=595, pady=15, sticky=tkinter.W)
        radio = tkinter.Radiobutton(master, text='Auto', variable = self.iTagStunde, value = 2, state=DISABLED, command=self.cmdRadio)
        radio.grid(row=1, column=1, padx=640, pady=15, sticky=tkinter.W)
        self.bTemp = tkinter.BooleanVar()
        self.bTemp.set(True)
        self.bDruck = tkinter.BooleanVar()
        self.bDruck.set(True)
        self.bFeuchte = tkinter.BooleanVar()
        self.bFeuchte.set(True)
        checkTemp = tkinter.Checkbutton(master, text='Temperatur', variable = self.bTemp, command=self.cmdCheck)
        checkTemp.grid(row=1, column=0, padx=15, pady=0, sticky=tkinter.W)
        checkDruck = tkinter.Checkbutton(master, text='Druck', variable = self.bDruck, command=self.cmdCheck)
        checkDruck.grid(row=1, column=0, padx=101, pady=0, sticky=tkinter.W)
        checkFeuchte = tkinter.Checkbutton(master, text='Feuchte', variable = self.bFeuchte, command=self.cmdCheck)
        checkFeuchte.grid(row=1, column=0, padx=156, pady=0, sticky=tkinter.W)
        
        self.cmdAktuell()
                
        self.ser = serial.Serial()
        self.ser.port = self.COMStr
        self.ser.baudrate = 9600
        self.ser.bytesize = serial.EIGHTBITS
        self.ser.parity = serial.PARITY_NONE
        self.ser.stopbits = serial.STOPBITS_ONE
        self.ser.timeout = 1
        self.ser.xonxoff = False
        self.ser.rtscts = False
        self.ser.dsrdtr = False
        self.ser.writeTimeout = 2

                
    def __del__(self):
        pass
    
    def setDatumEntry(self, startDate, endDate):
        self.eEnd.delete(0, 'end')
        self.eStart.delete(0, 'end')
        self.eEnd.insert(0, endDate.strftime("%Y-%m-%d %H:%M"))
        self.eStart.insert(0, startDate.strftime("%Y-%m-%d %H:%M"))
        
    def getDatumEntry(self):
        startDate = datetime.datetime.strptime(self.eStart.get(), "%Y-%m-%d %H:%M")
        endDate = datetime.datetime.strptime(self.eEnd.get(), "%Y-%m-%d %H:%M")
        return startDate, endDate
    
    def cmdBeschreibung(self):
        BeschreibungDialog = mytools.CBeschreibungDialog(self.master)
        self.master.wait_window(BeschreibungDialog.top)
        #print(mytools.decodeBeschreibung(BeschreibungDialog.result))
        if (BeschreibungDialog.result == -1): return
        self.cmdWetterLaden()
        time.sleep(2)
        try:
            self.ser.open()
        except IOError:
            tkinter.messagebox.showerror("serial port", "serial port kann nicht geoeffnet werden!")
        if self.ser.isOpen():
            try:
                self.ser.flushInput()
                self.ser.flushOutput()
                #write data
                self.ser.write('F'.encode('utf-8'))
                print("write data: F")
                time.sleep(0.5)
                #read data
                response = self.ser.readline()
                self.ser.close()
                #print(mytools.formatLine(response)+str(BeschreibungDialog.result).zfill(2)) 
                self.text.insert("end", mytools.formatLine(response)+str(BeschreibungDialog.result).zfill(2)+"\n")
            except IOError:
                tkinter.messagebox.showerror("serial port", "Kommunikationfehler!")
        else:
            tkinter.messagebox.showerror("serial port", "serial port kann nicht geoeffnet werden!")

    def cmdRadio(self):
        startDate, endDate = self.getDatumEntry()
        self.dateIntervall(startDate, endDate)

    def cmdCheck(self):
        startDate, endDate = self.getDatumEntry()
        self.dateIntervall(startDate, endDate)

    def cmdDatumOk(self):
        startDate, endDate = self.getDatumEntry()
        self.dateIntervall(startDate, endDate)

    def cmdSkipRueck(self):
        startDate = datetime.datetime.strptime(self.eStart.get(),"%Y-%m-%d %H:%M")+datetime.timedelta(-9)
        endDate = datetime.datetime.strptime(self.eEnd.get(),"%Y-%m-%d %H:%M")+datetime.timedelta(-9)
        self.setDatumEntry(startDate, endDate)
        self.dateIntervall(startDate, endDate)

    def cmdSkipVor(self):
        startDate = datetime.datetime.strptime(self.eStart.get(),"%Y-%m-%d %H:%M")+datetime.timedelta(9)
        endDate = datetime.datetime.strptime(self.eEnd.get(),"%Y-%m-%d %H:%M")+datetime.timedelta(9)
        self.setDatumEntry(startDate, endDate)
        self.dateIntervall(startDate, endDate)

    def cmdAktuell(self):
        endDate = datetime.datetime.now()
        startDate = endDate - datetime.timedelta(10)
        self.setDatumEntry(startDate, endDate)
        self.dateIntervall(startDate, endDate)

    def drawGraphik(self, xdate, ydate, ydate1, ydate2, startDate, endDate):
        f = Figure(figsize=(16,8), dpi=50)
        host = f.add_subplot(111)
        f.subplots_adjust(left=0.08, right=0.85, bottom=0.2)
        
        par1 = host.twinx()
        par2 = host.twinx()
        par2.spines["right"].set_position(("axes", 1.1))
        par2.spines["right"].set_visible(True)
        if (self.bTemp.get()):
            p1, = host.plot(xdate, ydate, "r-", label="Temperatur")
        else:
            p1, = host.plot(0, 0, "r-", label="Temperatur")
        if (self.bFeuchte.get()):
            p2, = par1.plot(xdate, ydate1, "b-", label="Feuchte")
        else:
            p2, = par1.plot(0, 0, "b-", label="Feuchte")
        if (self.bDruck.get()):
            p3, = par2.plot(xdate, ydate2, "g-", label="Druck")
        else:
            p3, = par2.plot(0, 0, "g-", label="Druck")

        host.xaxis.set_major_formatter(mdates.DateFormatter('%a %d.%m.%Y %H:%M'))
        host.set_xlim(startDate, endDate)
        f.autofmt_xdate(rotation=30, ha='center')
        host.set_yticks([-30, -20, -10, 0, 10, 20, 30 ,40 ,50, 60, 70])
        par1.set_yticks([0, 10, 20, 30 ,40 ,50, 60, 70, 80, 90, 100])
        par2.set_yticks([950, 960, 970, 980, 990, 1000, 1010, 1020, 1030, 1040, 1050])

        host.set_xlabel("Zeit")
        host.set_ylabel("Temperatur")
        par1.set_ylabel("Feuchte")
        par2.set_ylabel("Druck")

        host.yaxis.label.set_color(p1.get_color())
        par1.yaxis.label.set_color(p2.get_color())
        par2.yaxis.label.set_color(p3.get_color())

        tkw = dict(size=4, width=1.5)
        host.tick_params(axis='y', colors=p1.get_color(), **tkw)
        par1.tick_params(axis='y', colors=p2.get_color(), **tkw)
        par2.tick_params(axis='y', colors=p3.get_color(), **tkw)
        host.tick_params(axis='x', **tkw)
        host.grid(True) 
        
        canvas = FigureCanvasTkAgg(f, self.master)
        canvas.show()
        canvas.get_tk_widget().grid(row=0, column=1, padx=5, pady=5, sticky=tkinter.W)
        #toolbar = NavigationToolbar2TkAgg(canvas, self.master)
        #toolbar.update()
        
    def dateIntervall(self, startDate, endDate):
        xdate = []
        ytemp = []
        yfeuchte = []
        ydruck = []

        sql = "SELECT * FROM wetter WHERE datetime >= "+"'"+startDate.strftime("%Y-%m-%d %H:%M")+"'"+" AND datetime <= "+"'"+endDate.strftime("%Y-%m-%d %H:%M")+"'"
        cursor.execute(sql)
        if(self.iTagStunde.get() == 1):
            for row in cursor:
                xdate.append(datetime.datetime.strptime(row[0],"%Y-%m-%d %H:%M"))
                ytemp.append(row[1])
                yfeuchte.append(row[2])
                ydruck.append(row[3])
        else:
            row = cursor.fetchone()
            ndate = datetime.datetime.strptime(row[0],"%Y-%m-%d %H:%M").date()
            n=0; ntemp=0; nfeuchte=0; ndruck=0
            while row:
                if (ndate == datetime.datetime.strptime(row[0],"%Y-%m-%d %H:%M").date()):
                    n = n + 1
                    ntemp = ntemp + row[1]
                    nfeuchte = nfeuchte + row[2]
                    ndruck = ndruck + row[3]
                else:
                    xdate.append(ndate)
                    ytemp.append(ntemp/n)
                    yfeuchte.append(nfeuchte/n)
                    ydruck.append(ndruck/n)
                    #print (ndate); print (n);  print (ntemp/n)
                    ndate = datetime.datetime.strptime(row[0],"%Y-%m-%d %H:%M").date()
                    n=1; ntemp=row[1]; nfeuchte=row[2]; ndruck=row[3]
                row = cursor.fetchone()
            xdate.append(ndate)
            ytemp.append(ntemp/n)
            yfeuchte.append(nfeuchte/n)
            ydruck.append(ndruck/n)
            #print (ndate); print (n);  print (ntemp/n)
        connection.commit()
        self.drawGraphik(xdate, ytemp, yfeuchte, ydruck, startDate, endDate)     

    def fillMenuBar(self):
        self.menuBearbeiten = tkinter.Menu(self.menuBar, tearoff=False)
        self.menuDatei = tkinter.Menu(self.menuBar, tearoff=False)
        self.menuOptionen = tkinter.Menu(self.menuBar, tearoff=False)  
        self.menuBearbeiten.add_command(label="Wetterdaten aus Station laden", command=self.cmdWetterLaden)
        self.menuBearbeiten.add_command(label="Aktuelle Wetterdaten von Station", command=self.cmdAktuellLaden)
        self.menuBearbeiten.add_command(label="Wetterdaten in Datei speichern", command=self.cmdWetterSpeichern)
        self.menuBearbeiten.add_command(label="Wetterdaten auf Station loeschen", command=self.cmdWetterLoeschen)
        self.menuBearbeiten.add_command(label="Statistik", command=self.cmdWetterStatistik)
        self.menuBearbeiten.add_command(label="Wetterbeschreibung abrufen", command=self.cmdWetterBeschreibung)
        self.menuDatei.add_command(label="Wetterdaten aus Datei laden", command=self.cmdDateiLaden)
        self.menuDatei.add_command(label="Speichern", command=self.cmdDateiSpeichern, state=DISABLED)
        self.menuDatei.add_separator()
        self.menuDatei.add_command(label="Beenden", command=root.destroy)
        self.menuOptionen.add_command(label="Einstellungen", command=self.cmdEinstellungen)
        self.menuOptionen.add_command(label="Zeit einstellen", command=self.cmdZeit)             
        self.menuBar.add_cascade(label="Datei", menu=self.menuDatei)
        self.menuBar.add_cascade(label="Wetterdaten", menu=self.menuBearbeiten)
        self.menuBar.add_cascade(label="Optionen", menu=self.menuOptionen)
        
    def cmdWetterBeschreibung(self):
        def cmdOK():
            strDate = eDate.get()
            strDate1 = strDate + " 00:00"
            strDate2 = strDate + " 23:59"
            #print(strDate1)
            #print(strDate2)
            T.delete("1.0", "end")
            sql = "SELECT * FROM wetter WHERE datetime >= "+"'"+strDate1+"'"+" AND datetime <= "+"'"+strDate2+"'"
            cursor.execute(sql)
            res = cursor.fetchone()
            while res:
                if (res[4] != ''):
                    lst = mytools.decodeBeschreibung(res[4])
                    #print(res[4])
                    #print(lst)
                    T.insert(tkinter.END, res[0]+"\n"+"Temperatur = "+str(res[1])+"\n"+"Feuchte = "+str(res[2])+" %\n"+"Druck = "+str(res[3])+" hPa\n"+lst[0]+"\n"+lst[1]+"\n"+lst[2]+"\n"+lst[3]+"\n\n")
                res = cursor.fetchone()
            
        d = tkinter.Tk()
        d.minsize(320, 200)
        d.title("Wetterbeschreibung")
        tkinter.Label(d, text="Datum:").grid(row=0, column=0, padx=5, pady=5, sticky=tkinter.W)
        eDate = tkinter.Entry(d, width=15)
        T = tkinter.scrolledtext.ScrolledText(d, height=8, width=19)
        T.grid(row=1, column=0, padx=5, pady=0, sticky=tkinter.W)
        eDate.grid(row=0, column=0, padx=57, pady=5, sticky=tkinter.W)
        Date = datetime.datetime.now()
        eDate.insert(0, Date.strftime("%Y-%m-%d"))
        tkinter.Button(d, text='OK', command=cmdOK).grid(row=0, column=1, padx=5, pady=5, sticky=tkinter.W)
        tkinter.Button(d, text='Abbruch',  command=d.destroy).grid(row=0, column=2, padx=5, pady=5, sticky=tkinter.W)
        
    def cmdWetterStatistik(self):
        strDate1 = "2017-01-01"
        strDate2 = "2017-01-31"
        
        def calStatistik(strDate1, strDate2):
            sql = "SELECT AVG(temperatur) FROM wetter WHERE datetime >= "+"'"+strDate1+"'"+" AND datetime <= "+"'"+strDate2+"'"
            cursor.execute(sql)
            res = cursor.fetchone()
            #print(res) 
            sql = "SELECT datetime, MAX(temperatur) FROM wetter WHERE datetime >= " + "'" + strDate1 + "'" + " AND datetime <= " + "'" + strDate2 + "'"
            cursor.execute(sql)
            res1 = cursor.fetchall()
            #print(res1) 
            sql = "SELECT datetime, MIN(temperatur) FROM wetter WHERE datetime >= " + "'" + strDate1 + "'" + " AND datetime <= " + "'" + strDate2 + "'"
            cursor.execute(sql)
            res2 = cursor.fetchall()
            #print(res2) 
            tkinter.Label(d3, text= "Mittl. Temperatur =").grid(row=4,column=0,padx=5, pady=2, sticky=tkinter.W)
            tkinter.Label(d3, text= str(format(res[0], ".2f")) + " C").grid(row=4,column=1,padx=5, pady=2, sticky=tkinter.W)
            tkinter.Label(d3, text= "Max. Temperatur =").grid(row=5,column=0,padx=5, pady=2, sticky=tkinter.W)
            tkinter.Label(d3, text= res1[0][0]+"  "+str(res1[0][1])+" C").grid(row=5,column=1,padx=5, pady=2, sticky=tkinter.W)
            tkinter.Label(d3, text= "Min. Temperatur =").grid(row=6,column=0,padx=5, pady=2, sticky=tkinter.W)
            tkinter.Label(d3, text= res2[0][0]+"  "+str(res2[0][1])+" C").grid(row=6,column=1,padx=5, pady=2, sticky=tkinter.W)
        
        def cmdStatistik():
            strDate1 = date1.get()
            strDate2 = date2.get()
            calStatistik(strDate1, strDate2)
            
        def cmdMinusJahr():
            startDate = datetime.datetime.strptime(date1.get(), "%Y-%m-%d")-relativedelta(years=1)
            endDate = datetime.datetime.strptime(date2.get(), "%Y-%m-%d")-relativedelta(years=1)
            strDate1=startDate.strftime("%Y-%m-%d")
            strDate2=endDate.strftime("%Y-%m-%d")
            date1.delete(0, 'end'); date1.insert(0, strDate1)
            date2.delete(0, 'end'); date2.insert(0, strDate2)
            calStatistik(strDate1, strDate2)
            
        def cmdPlusJahr():
            startDate = datetime.datetime.strptime(date1.get(), "%Y-%m-%d")+relativedelta(years=1)
            endDate = datetime.datetime.strptime(date2.get(), "%Y-%m-%d")+relativedelta(years=1)
            strDate1=startDate.strftime("%Y-%m-%d")
            strDate2=endDate.strftime("%Y-%m-%d")
            date1.delete(0, 'end'); date1.insert(0, strDate1)
            date2.delete(0, 'end'); date2.insert(0, strDate2)
            calStatistik(strDate1, strDate2)
            
        def cmdMinusMonat():
            startDate = datetime.datetime.strptime(date1.get(), "%Y-%m-%d")-relativedelta(months=1)
            endDate = datetime.datetime.strptime(date2.get(), "%Y-%m-%d")-relativedelta(months=1)
            strDate1=startDate.strftime("%Y-%m-%d")
            strDate2=endDate.strftime("%Y-%m-%d")
            date1.delete(0, 'end'); date1.insert(0, strDate1)
            date2.delete(0, 'end'); date2.insert(0, strDate2)
            calStatistik(strDate1, strDate2)
            
        def cmdPlusMonat():
            startDate = datetime.datetime.strptime(date1.get(), "%Y-%m-%d")+relativedelta(months=1)
            endDate = datetime.datetime.strptime(date2.get(), "%Y-%m-%d")+relativedelta(months=1)
            strDate1=startDate.strftime("%Y-%m-%d")
            strDate2=endDate.strftime("%Y-%m-%d")
            date1.delete(0, 'end'); date1.insert(0, strDate1)
            date2.delete(0, 'end'); date2.insert(0, strDate2)
            calStatistik(strDate1, strDate2)                  
   
        d3 = tkinter.Tk()
        d3.minsize(320, 200)
        d3.title("Satistik")
        tkinter.Label(d3, text="Startdatum:").grid(row=0, padx=5, pady=5, sticky=tkinter.W)
        date1 = tkinter.Entry(d3, width=20)
        tkinter.Label(d3, text="Enddatum:").grid(row=1 ,padx=5, pady=5, sticky=tkinter.W)
        date2 = tkinter.Entry(d3, width=20)
        date1.grid(row=0,column=1,padx=5, pady=5, sticky=tkinter.W)
        date2.grid(row=1,column=1,padx=5, pady=5, sticky=tkinter.W)
        date1.insert(0, strDate1)
        date2.insert(0, strDate2)
        tkinter.Button(d3, text='Berechne Statistik', command=cmdStatistik).grid(row=3, column=0, padx=5, pady=10, sticky=tkinter.W)
        #tkinter.Button(d3, text='Ende', command=d3.destroy).grid(row=3,column=1,padx=5, pady=10, sticky=tkinter.W)
        tkinter.Button(d3, text='<<', command=cmdMinusJahr).grid(row=3, column=1, padx=5, pady=10, sticky=tkinter.W)
        tkinter.Button(d3, text='>>', command=cmdPlusJahr).grid(row=3, column=1, padx=35, pady=10, sticky=tkinter.W)
        tkinter.Button(d3, text='<', command=cmdMinusMonat).grid(row=3, column=1, padx=65, pady=10, sticky=tkinter.W)
        tkinter.Button(d3, text='>', command=cmdPlusMonat).grid(row=3, column=1, padx=87, pady=10, sticky=tkinter.W)

    def cmdZeit(self):
        d1 = tkinter.Tk()
        d1.title("Zeit einstellen")
        tkinter.Label(d1, text="Datum / Zeit:  ").pack(padx=5, pady=5)
        e1 = tkinter.Entry(d1, width=20)
        def cmdZeitsetzen():
            strg=e1.get()
            strg1='B'+strg[0:2]+strg[3:5]+strg[8:10]+strg[11:13]+strg[14:16]
            print(strg1) 
            try:
                self.ser.open()
            except IOError:
                tkinter.messagebox.showerror("serial port", "serial port kann nicht geoeffnet werden!")
            if self.ser.isOpen():
                try:
                    self.ser.flushInput()
                    self.ser.flushOutput()
                    #write data
                    self.ser.write(strg1.encode('utf-8'))
                    time.sleep(0.2)
                    self.ser.close()
                except IOError:
                    tkinter.messagebox.showerror("serial port", "Kommunikationfehler!")
            else:
                tkinter.messagebox.showerror("serial port", "serial port kann nicht geoeffnet werden!")
            d1.destroy() 
        e1.pack(padx=5, pady=5)
        dt = datetime.datetime.now()
        e1.insert(20, dt.strftime("%d.%m.%Y %H:%M"))
        tkinter.Button(d1, text='Abbruch', command=d1.destroy).pack(side="right", padx=10, pady=10)
        tkinter.Button(d1, text='Uebernehmen', command=cmdZeitsetzen).pack(side="left", padx=10, pady=10)
        
    def cmdAktuellLaden(self):
        try:
            self.ser.open()
        except IOError:
            tkinter.messagebox.showerror("serial port", "serial port kann nicht geoeffnet werden!")
        if self.ser.isOpen():
            try:
                self.ser.flushInput()
                self.ser.flushOutput()
                #write data
                self.ser.write('F'.encode('utf-8'))
                print("write data: F")
                time.sleep(0.5)
                #read data
                response = self.ser.readline()
                self.ser.close()
                tkinter.messagebox.showinfo("Aktuelle Wetterdaten", mytools.formatLine(response)) 
            except IOError:
                tkinter.messagebox.showerror("serial port", "Kommunikationfehler!")
        else:
            tkinter.messagebox.showerror("serial port", "serial port kann nicht geoeffnet werden!")
                         
    def cmdWetterLaden(self):
        self.text.delete("1.0", "end")
        try:
            self.ser.open()
        except IOError:
            tkinter.messagebox.showerror("serial port", "serial port kann nicht geoeffnet werden!")
        if self.ser.isOpen():
            try:
                self.ser.flushInput()
                self.ser.flushOutput()
                #write data
                self.ser.write('A'.encode('utf-8'))
                print("write data: A")
                time.sleep(0.2)
                #read data
                while self.ser.inWaiting():
                    response = self.ser.readline()
                    self.text.insert("end", mytools.formatLine(response)+"\n")
                    time.sleep(0.1) 
                    root.update() 
                    #if not ser.inWaiting(): break
                self.ser.close()
            except IOError:
                tkinter.messagebox.showerror("serial port", "Kommunikationfehler!")
        else:
            tkinter.messagebox.showerror("serial port", "serial port kann nicht geoeffnet werden!")

    def cmdWetterSpeichern(self):
        strTxt = self.text.get("1.0", "end")
        lst = strTxt.splitlines()
        cursor.execute('''SELECT MAX(datetime) FROM wetter''')
        maxDatetime=datetime.datetime.strptime(str(cursor.fetchone()[0]), "%Y-%m-%d %H:%M")
        for n in range(len(lst)-1):
            if(mytools.line2date(lst[n]) > maxDatetime):
                werte = mytools.line2date(lst[n]).strftime("%Y-%m-%d %H:%M"), mytools.line2temp(lst[n]), mytools.line2feuchte(lst[n]), mytools.line2druck(lst[n]), mytools.line2beobachtung(lst[n])
                sql = "INSERT INTO wetter VALUES (?, ?, ?, ?, ?)"
                cursor.execute(sql, werte)
        connection.commit()  

    def cmdWetterLoeschen(self):
        if tkinter.messagebox.askyesno("Wetter", "Daten auf Wetterstation loeschen?"):
            try:
                self.ser.open()
            except IOError:
                tkinter.messagebox.showerror("serial port", "serial port kann nicht geoeffnet werden!")
            if self.ser.isOpen():
                try:
                    self.ser.flushInput()
                    self.ser.flushOutput()
                    #write data
                    self.ser.write('D'.encode('utf-8'))
                    print("write data: D")
                    time.sleep(0.2)
                    self.ser.close()
                    tkinter.messagebox.showinfo("Wetter", "Wetterdaten wurden auf Station geloescht!")
                except IOError:
                    tkinter.messagebox.showerror("serial port", "Kommunikationfehler!")
            else:
                tkinter.messagebox.showerror("serial port", "serial port kann nicht geoeffnet werden!")

    def cmdDateiSpeichern(self):
        """
        f = open("c:\\technik\\wetterdaten.txt", "r")
        for Line in f:
            werte = mytools.line2date(Line).strftime("%Y-%m-%d %H:%M"), mytools.line2temp(Line), mytools.line2feuchte(Line), mytools.line2druck(Line), mytools.line2beobachtung(Line)
            sql = "INSERT INTO wetter VALUES (?, ?, ?, ?, ?)"
            cursor.execute(sql, werte)
        f.close()
        connection.commit()
        cursor.close()

        cursor.execute("SELECT * FROM wetter WHERE datetime > '2016-12-01 00:00'")
        for res in cursor:
            #a, b, c, d, e = res
            a, = res
            print(a)
            
        #cursor.execute('''SELECT max(datetime) FROM wetter''')
        #maxDatetime=datetime.datetime.strptime(cursor.fetchone()[0], '%Y-%m-%d %H:%M')
        #print(maxDatetime)
        connection.commit()
        #cursor.close()
        #connection.close()
        """

    def cmdDateiLaden(self):
        cursor.execute("SELECT * FROM wetter")
        self.text.delete("1.0", "end")
        for res in cursor:
            #a, b, c, d, e = res
            self.text.insert("end", res[0]+' '+str(res[1])+' '+str(res[2])+' '+str(res[3])+' '+str(res[4])+'\n')
        connection.commit()
        
    def cmdEinstellungen(self):
        d2 = tkinter.Tk()
        d2.minsize(320, 100)
        d2.title("Einstellungen")
        path = tkinter.Entry(d2, width=30)
        port = tkinter.Entry(d2, width=30)
        def cmdEinstSpeichern():
            self.PathStr = path.get()
            self.COMStr = port.get()
            config = {'key1': self.PathStr, 'key2': self.COMStr}
            with open('config.json', 'w') as f:
                json.dump(config, f)
            d2.destroy()    
        tkinter.Label(d2, text="Pfad Wetterdaten:  ").grid(row=0, column=0,padx=5, pady=5, sticky=tkinter.W)
        tkinter.Label(d2, text="COM Port:  ").grid(row=1,column=0, padx=5, pady=5, sticky=tkinter.W)
        path.grid(row=0,column=1,padx=5, pady=5)
        port.grid(row=1,column=1,padx=5, pady=5)
        path.insert(0, self.PathStr)
        port.insert(0, self.COMStr)
        tkinter.Button(d2, text='Uebernehmen', command=cmdEinstSpeichern).grid(row=2, column=0,padx=5, pady=10, sticky=tkinter.W)
        tkinter.Button(d2, text='Abbruch', command=d2.destroy).grid(row=2, column=1, padx=5, pady=10, sticky=tkinter.W)

PathStr, COMStr = loadSetup()
connection = sqlite3.connect(PathStr) 
cursor = connection.cursor()

root = tkinter.Tk()
root.title("Wetterstation 1.0 by www.bitpilot.de")
root.maxsize(1200,600)
app = WetterApp(root)
app.mainloop()

cursor.close()
connection.close()
